/*****************************************************************************************
**
** vEnglish - French-English Dictionary and more !
** Contact : vin.fouache@free.fr
**
** Copyright (C) 2008  Vincent Fouache
**
** This file is part of vEnglish.
**
** vEnglish is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation, either version 3 of the License, or
** (at your option) any later version.

** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>
**
***************************************************************************************/


/* ********** ********** ********** ********** ********** ********* *\
*
*               FenAbout.cc
*               -----------
*
*   Auteur : Vincent Fouache
*   Date de création : 24 Octobre 2008
*   Dernières modifications : 09/12/08
*
*   Description : Implémentation de la classe FenAbout
*
\* ********** ********** ********** ********** ********** ********* */

#include "FenAbout.hh"


FenAbout::FenAbout(QWidget *parent = 0) : QDialog(parent)
{
    // Création des différents onglets
    ongletInformations = new QTextEdit(this);
    ongletProprietes = new QLabel(this);
    ongletLicence = new QWidget(this);

    creerOngletInformations();
    creerOngletProprietes();
    creerOngletLicence();

    // Création du conteneur des onglets
    creerConteneur();

    // Quelques propriétés pour la fenêtre
    setLayout(layoutPrincipal);
    setWindowTitle("vEnglish - A propos...");
    setFixedHeight(355);
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

void FenAbout::creerOngletInformations()
{
	QString informations;
    informations += "Un certain jeudi 23 octobre, une idée assez bizarre a germé dans ma tête. ";
    informations += "Je voulais stocker mon vocabulaire d'anglais appris en cours sur mon PC...<br />";
    informations += "J'ai donc décidé de me créer un petit logiciel qui me gèrerait un dictionnaire Français/Anglais. ^^<br />";
    informations += "vEnglish était né !";

    informations +="<br /><br />";

    informations +="<b>Notes</b> : Les icônes sont sous la LGPL et viennent du <u>Crystal Project Icons</u> par <i>Everaldo Coelho</i> (http://www.everaldo.com).";

    ongletInformations->setStyleSheet("QTextEdit { margin: 10px; padding: 8px; }");
    ongletInformations->setText(informations);
    ongletInformations->setReadOnly(true);
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

void FenAbout::creerOngletProprietes()
{
    QString proprietes;
    proprietes += "<b>Nom :</b> vEnglish<br /><br />";
    proprietes += "<b>Version :</b> 0.6.0<br /><br />";
    proprietes += "<b>Licence :</b> GNU GPL v3<br /><br />";
    proprietes += "<b>Création :</b> 23 Octobre 2008<br /><br />";
    proprietes += "<b>Développeur :</b> <a href='mailto:vin.fouache@free.fr'>Vincent Fouache</a>";
    proprietes += "&nbsp;&nbsp;&nbsp;&nbsp;";
    proprietes += "( <a href='http://vince.weblog.free.fr'>www</a> )";

    ongletProprietes->setStyleSheet("QLabel { margin: 10px; }");
    ongletProprietes->setOpenExternalLinks(true);
    ongletProprietes->setText(proprietes);
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

void FenAbout::creerOngletLicence()
{
	// Texte
    QTextEdit *licenceTexte = new QTextEdit(this);
    QString licenceStr = "<center><b>vEnglish</b><br />French-English Dictionary and more !<br />";
    licenceStr += "Contact : <i>vin.fouache@free.fr</i><br />";
    licenceStr += "<b>Copyright (C) 2008  Vincent Fouache</b><br /></center>";
    licenceStr += "vEnglish is free software: you can redistribute it and/or modify";
    licenceStr += "it under the terms of the GNU General Public License as published by";
    licenceStr += "the Free Software Foundation, either version 3 of the License, or";
    licenceStr += "(at your option) any later version.";
    licenceStr += "<br /><br />";
    licenceStr += "This program is distributed in the hope that it will be useful,";
    licenceStr += "but WITHOUT ANY WARRANTY; without even the implied warranty of";
    licenceStr += "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the";
    licenceStr += "GNU General Public License for more details.";
    licenceStr += "<br /><br />";
    licenceStr += "You should have received a copy of the GNU General Public License";
    licenceStr += "along with this program.  If not, see http://www.gnu.org/licenses/";

    licenceTexte->setText(licenceStr);
    licenceTexte->setReadOnly(true);
    licenceTexte->setStyleSheet("QTextEdit { padding: 6px; }");

    // Logo licence
    QLabel *logo = new QLabel("<a href='http://www.gnu.org/licenses/'><img src=':/images/licence.png' /></a>", this);
    logo->setOpenExternalLinks(true);

    // Pour centrer le logo
    QHBoxLayout *logoLayout = new QHBoxLayout;
    logoLayout->addWidget(logo);
    logoLayout->setAlignment(Qt::AlignHCenter);

	// Layout
    QVBoxLayout *licenceLayout = new QVBoxLayout;
    licenceLayout->addWidget(licenceTexte);
    licenceLayout->addLayout(logoLayout);
	licenceLayout->setAlignment(Qt::AlignVCenter);

    ongletLicence->setLayout(licenceLayout);
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

void FenAbout::creerConteneur()
{
    // Création du QTabWidget
    QTabWidget *onglets = new QTabWidget(this);

    // Ajout des onglets au QTabWidget
	onglets->addTab(ongletInformations, "Informations");
	onglets->addTab(ongletProprietes, "Propriétés");
	onglets->addTab(ongletLicence, "Licence");

	// Le titre
	QLabel *titre = new QLabel("<h1>vEnglish</h1>", this);
    titre->setAlignment(Qt::AlignHCenter);

	// L'icone
    QLabel *icone = new QLabel(this);
    icone->setPixmap(QPixmap(":/images/tudor_rose.png"));
    icone->setAlignment(Qt::AlignHCenter);

    // Dernière compilation
    QLabel *lastBuild = new QLabel("<span style='font-size: small;'><i>Compilé le 09 Décembre 2008.</i></span>", this);
    lastBuild->setAlignment(Qt::AlignRight);

    // Le bouton fermer
    QPushButton *fermer = new QPushButton("&Fermer", this);
    connect(fermer, SIGNAL(clicked()), this, SLOT(accept()));

    // Création du layout proncipal
    layoutPrincipal = new QVBoxLayout;
    layoutPrincipal->addWidget(titre);
    layoutPrincipal->addWidget(icone);
    layoutPrincipal->addWidget(lastBuild);
	layoutPrincipal->addWidget(onglets);
	layoutPrincipal->addWidget(fermer, 0, Qt::AlignRight);
}
