/*****************************************************************************************
**
** vEnglish - French-English Dictionary and more !
** Contact : vin.fouache@free.fr
**
** Copyright (C) 2008  Vincent Fouache
**
** This file is part of vEnglish.
**
** vEnglish is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation, either version 3 of the License, or
** (at your option) any later version.

** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>
**
***************************************************************************************/


/*********** ********** ********** ********** ********** **********\
*
*               FenInterro.cc
*               -------------
*
*   Auteur : Vincent Fouache
*   Date de création : 29 Octobre 2008
*   Dernières modifications : 09/12/2008
*
*   Description : Implémentation de la classe FenInterro
*
\*********** ********** ********** ********** ********** **********/

#include "FenInterro.hh"


FenInterro::FenInterro(const Dicos &dicosP, const Preferences &prefP)
{
    // Récupération du dico
    dicos = dicosP;
    // Récupération des préférences
    pref = prefP;

    // L'interro n'est pas encore corrigé évidemment...
    isCorriger = false;

    creerInterro();

    // Les boutons
    boutonCorriger = new QPushButton("&Corriger", this);
    connect(boutonCorriger, SIGNAL(clicked()), this, SLOT(corriger()));
    boutonAbandonner = new QPushButton("&Abandonner", this);
    connect(boutonAbandonner, SIGNAL(clicked()), this, SLOT(accept()));

    // Note et commentaire
    labelNote = new QLabel(this);
    labelNote->setVisible(false);
    labelCommentaire = new QLabel(this);
    labelCommentaire->setVisible(false);

    // On crée un layout à part pour centrer la note
    QHBoxLayout *layoutNote = new QHBoxLayout;
    layoutNote->addWidget(labelNote);
    layoutNote->setAlignment(Qt::AlignHCenter);

    // On crée un layout à part pour centrer la note
    QHBoxLayout *layoutcommentaire = new QHBoxLayout;
    layoutcommentaire->addWidget(labelCommentaire);
    layoutcommentaire->setAlignment(Qt::AlignHCenter);

    // Pour centrer les boutons
    QHBoxLayout *layoutBoutons = new QHBoxLayout;
    layoutBoutons->addWidget(boutonAbandonner);
    layoutBoutons->addWidget(boutonCorriger);

    // On crée une zone avec scroll pour les interros qui comportent trop de mots
    QScrollArea *interroScroll = new QScrollArea;
    interroScroll->setWidget(interro);

	// Layout Principal
    QVBoxLayout *layoutPrincipal = new QVBoxLayout;
    layoutPrincipal->addWidget(interroScroll);
    layoutPrincipal->addLayout(layoutNote);
    layoutPrincipal->addLayout(layoutcommentaire);
    layoutPrincipal->addLayout(layoutBoutons);

    setWindowTitle("vEnglish - Interrogation");
    setLayout(layoutPrincipal);

    // focus sur le premier QLineedit
    reponses[0].setFocus();
}

void FenInterro::creerInterro()
{
    // Création du conteneur
    interro = new QWidget(this);

    // Initialisations
    srand(time(NULL));
    const int MIN = 0;
    const int MAX = dicos.taille() - 1; // on enlève 1, car les lists commencent à 0...
    int alea;

    icones = new QLabel[pref.getNbMots()];
    questions = new QLabel[pref.getNbMots()];
    reponses = new QLineEdit[pref.getNbMots()];
    corrections = new QLabel[pref.getNbMots()];

    // On crée une list qui contiendra toutes les nb aléatoires générés
    QList<int> saveAlea;

    // On remplit les tableaux de questions de façon aléatoire
    for (int i = 0 ; i < pref.getNbMots() ; i++)
    {
        do
        {
            alea = (rand() % (MAX - MIN + 1)) + MIN;
        }
        while (saveAlea.indexOf(alea) != -1);

        // On enregistre le nb aléatoire
        saveAlea.append(alea);


        // On assigne un mot du dictionnaire à la question
        // Puis sa traduction à "correction"
        if (pref.getLangQuestions() == FR)
        {
            (questions[i]).setText(dicos.getFr(alea));
            (corrections[i]).setText(dicos.getEn(alea));
        }
        else if (pref.getLangQuestions() == EN)
        {
            (questions[i]).setText(dicos.getEn(alea));
            (corrections[i]).setText(dicos.getFr(alea));
        }
        else
        {
            int alea2 = (rand() % (2)); // Aléatoire : 0 ou 1
            if (alea2 == 0)
            {
                (questions[i]).setText(dicos.getFr(alea));
                (corrections[i]).setText(dicos.getEn(alea));
            }
            else
            {
                (questions[i]).setText(dicos.getEn(alea));
                (corrections[i]).setText(dicos.getFr(alea));
            }
        }


        // On met la correction en vert
        (corrections[i]).setStyleSheet("QLabel { color: green; }");

        // On masque l'icone et la correction
        (icones[i]).setVisible(false);
        (corrections[i]).setVisible(false);
    }

    // Génération du layout
    QGridLayout *layoutInterro = new QGridLayout;
    for (int i = 0 ; i < pref.getNbMots() ; i++)
    {
        layoutInterro->addWidget(&icones[i], i , 0);
        layoutInterro->addWidget(&questions[i], i, 1);
        layoutInterro->addWidget(&reponses[i], i, 2);
        layoutInterro->addWidget(&corrections[i], i, 4);
    }

    // Assignation du layout au widget conteneur
    interro->setLayout(layoutInterro);
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

void FenInterro::keyPressEvent(QKeyEvent *event)
{
    // Les 2 touches "Entrée"
    if (event->key() == Qt::Key_Enter || event->key() == Qt::Key_Return)
        if (!isCorriger)
            corriger();
        else
            accept();
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

void FenInterro::corriger()
{
    bool continuer = true;
    isCorriger = true;
    float note = 0.00;
    QString commentaire = "";

    // Si la confirmation "l'interro n'est pas finie" est activé
    if (pref.getConfirmSiPasFinie())
    {
        // L'utilisateur a-t-il répondu à toutes les questions ?
        for (int i = 0 ; i < pref.getNbMots() && continuer ; i++)
        {
            // Si le champ de la réponse n°i est vide
            if ( ((reponses[i]).text()).isEmpty() )
            {
                // On arrète la boucle
                continuer = false;
                // et on demande confirmation
                int reponse = QMessageBox::question(this, "Confirmation", "Vous n'avez pas pas fini l'interrogation.<br />Voulez-vous quand même la corriger ?", QMessageBox::Yes | QMessageBox::No);
                if (reponse == QMessageBox::No)
                    isCorriger = false;
            }
        }
    }

    if (isCorriger)
    {
        // On corrige
        for (int i = 0 ; i < pref.getNbMots() ; i++)
        {
            // Création de la nouvelle réponse en fonction des options
            // On ne modifie pas directement "reponse" pour qu'il n'y ait aucun impact sur l'affichage
            QString newReponse = (reponses[i]).text();
            QString newCorrection = (corrections[i]).text();

            // Si on ne prend pas en compte la casse dans la correction
            // Attention, cette condition doit être avant les autres !
            if (!pref.getResperterCasse())
            {
                newReponse.toLower();
                newCorrection.toLower();
            }

            // Si on ne prend pas en compte les "to " dans la correction,
            // que la correction commence par "to "
            // et que la réponse ne commence pas par "to "
            // alors on ajoute "to " à la réponse
            if ( pref.getToExclus() && newCorrection.startsWith("to ") && !(newReponse.startsWith("to ")) )
                    newReponse.insert(0, "to ");

            // Si on ne prend pas en compte le contenu des () situées à la fin des expressions
            if ( pref.getParenthesesExclues() && newCorrection.indexOf('(', 0) != -1 )
            {
                    // Si l'utilisateur a mis le contenu exact des ()
                    // alors c'est pas la peine de faire ce qui suit...
                    if (newReponse != newCorrection)
                    {
                    int indiceDebut = newCorrection.indexOf('(', 0);
                    int indiceFin = newCorrection.indexOf(')', indiceDebut);

                    // Si dans le dico, il y a un espace avant la (
                    // et qu'il n'y est pas à la fin de la réponse
                    // alors on le rajoute
                    if (newCorrection[indiceDebut - 1] == ' ' && newReponse[indiceDebut - 1] != ' ')
                    {
                        newReponse.insert(indiceDebut - 1, ' ');
                        indiceFin++;
                    }

                    // On enlève les () et leur contenu.
                    newCorrection.remove(indiceDebut, indiceFin);
                }
            }

            // Si la réponse est FAUSSE
            if (newReponse != newCorrection)
            {
                // On assigne une img no à l'icone.
                (icones[i]).setPixmap(QPixmap(":/images/icones/no.png"));

                // On affiche la correction.
                (corrections[i]).setVisible(true);

                // On met en rouge l'erreur.
                (reponses[i]).setStyleSheet("QLineEdit { color: red; }");
            }
            // Si c'est la bonne réponse
            else
            {
                // On assigne une img ok à l'icone.
                (icones[i]).setPixmap(QPixmap(":/images/icones/ok.png"));
                note++;
            }

            // On affiche l'icone.
            (icones[i]).setVisible(true);
        }

        // On enlève le bouton Corriger
        boutonCorriger->setVisible(false);
        // On renomme le bouton Abandonner en Fermer
        boutonAbandonner->setText("Fermer");

        // Calcul de la note sur p_noteSur
        note = note * pref.getNoteSur() / pref.getNbMots();

        // Troncature au centième
        note = (int (note * 100) / 100.00);
        // Arrondi au plus proche
        if ((note + 0.05) > (int (note * 10) / 10.0) + 0.1)
            note = (int (note * 10) / 10.0) + 0.1; // Troncature au dixième + 0.1
        else
            note = (int (note * 10) / 10.0); // Troncature au dixième

        // On affiche la note
        labelNote->setText("<b>Note : " + QString::number(note) + " / " + QString::number(pref.getNoteSur()) + "</b>");
        labelNote->setVisible(true);

        // Si le commentaire est activé, on l'affiche
        if (pref.getCommentaire())
        {
            // On prend le commentaire correspondant à la note.
            if (note == 0)
                commentaire = "Pouah t'es trop nul !!!";
            else if (note > 0 && note < pref.getNoteSur() / 2.5)
                commentaire = "Hé ben... Il y a des révisions à faire !";
            else if (note >= pref.getNoteSur() / 2.5 && note < pref.getNoteSur() / 2)
                commentaire = "Dommage, tu as presque la moyenne... Ne perds pas espoir !";
            else if (note == pref.getNoteSur() / 2)
                commentaire = "Tout juste la moyenne. C'est à peine acceptable, tu peux franchement faire mieux !";
            else if (note > pref.getNoteSur() / 2 && note < pref.getNoteSur() / 1.2)
                commentaire = "C'est moyen. J'espère que tu vas vite progresser !";
            else if (note >= pref.getNoteSur() / 1.2 && note < pref.getNoteSur())
                commentaire = "Bravo ! C'est bien, le 20 n'est plus très loin !";
            else if (note == pref.getNoteSur())
                commentaire = "Excellent. Rien à redire. Total respect.";
            else
                commentaire = "Heu... Désolé, je bug...";

            labelCommentaire->setText("<i>" + commentaire + "</i>");
            labelCommentaire->setVisible(true);
        }
    }
}
