/*****************************************************************************************
**
** vEnglish - French-English Dictionary and more !
** Contact : vin.fouache@free.fr
**
** Copyright (C) 2008  Vincent Fouache
**
** This file is part of vEnglish.
**
** vEnglish is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation, either version 3 of the License, or
** (at your option) any later version.

** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>
**
***************************************************************************************/


/* ********** ********** ********** ********** ********** ********* *\
*
*               FenRecherche.cc
*               ---------------
*
*   Auteur : Vincent Fouache
*   Date de création : 31 Octobre 2008
*   Dernières modifications : 05/11/08
*
*   Description : Implémentation de la classe FenPreferences
*
\* ********** ********** ********** ********** ********** ********* */

#include "FenPreferences.hh"


FenPreferences::FenPreferences(const Preferences &prefP, const int tailleDicoP, QWidget *parent = 0) : QDialog(parent)
{
    // Récupération des préférences
    pref = prefP;
    // et de la taille du dico
    tailleDico = tailleDicoP;

    ok = false;

    // Création des différents onglets
    ongletGeneral = new QWidget(this);
    ongletInterros = new QWidget(this);

    creerOngletGeneral();
    creerOngletInterros();

    // Création du conteneur des onglets
    creerConteneur();

    // Quelques propriétés pour la fenêtre
    setLayout(layoutPrincipal);
    setWindowTitle("vEnglish - Préférences");
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

void FenPreferences::creerOngletGeneral()
{
    // --- Affichage ---
    QGroupBox *groupAffichage = new QGroupBox("Affichage", this);

    colonneGauche = new QComboBox(this);
    colonneGauche->addItem("Français");
    colonneGauche->addItem("Anglais");
    if (pref.getColGauche() == EN)
        colonneGauche->setCurrentIndex(1);

    largeurColonne = new QSpinBox(this);
    largeurColonne->setRange(LGCOLMIN, LGCOLMAX);
    largeurColonne->setValue(pref.getLargeurColonnes());

    tailleTexte = new QComboBox(this);
    tailleTexte->addItem("Petit");
    tailleTexte->addItem("Normal");
    tailleTexte->addItem("Gros");
    tailleTexte->addItem("Très Gros");
    tailleTexte->setCurrentIndex(pref.getTailleTexte());

    QFormLayout *layoutAffichage = new QFormLayout;
    layoutAffichage->addRow("Colonne à gauche : ", colonneGauche);
    layoutAffichage->addRow("Largeur des colonnes : ", largeurColonne);
    layoutAffichage->addRow("Taille du texte", tailleTexte);

    groupAffichage->setLayout(layoutAffichage);



    // --- Tri ---
    QGroupBox *groupTri = new QGroupBox("Tri", this);

    triAuto = new QCheckBox("Trier automatiquement à l'ouverture", this);
    connect(triAuto, SIGNAL(clicked(bool)), this, SLOT(activerOrdre(bool)));

    ordre = new QComboBox(this);
    ordre->addItem("De A à Z");
    ordre->addItem("De Z à A");
    ordre->setSizeAdjustPolicy(QComboBox::AdjustToContents);
    if (pref.getTriOrdre() == DECROISSANT)
        ordre->setCurrentIndex(1);

    // Si le tri auto est activé
    if (pref.getTriAuto())
        triAuto->setChecked(true);
    // Sinon on désactive l'ordre de tri
    else
        ordre->setEnabled(false);


    respecterCasse = new QCheckBox("Respecter la casse", this);
    respecterCasse->setChecked(pref.getTriCasse());

    QVBoxLayout *layoutTri = new QVBoxLayout;
    layoutTri->addWidget(respecterCasse);
    layoutTri->addWidget(triAuto);
    layoutTri->addWidget(ordre, 0, Qt::AlignHCenter);

    groupTri->setLayout(layoutTri);



    // --- Divers ---
    QGroupBox *groupDivers = new QGroupBox("Divers", this);

    confirmSuppr = new QCheckBox("Confirmer la suppression d'une ligne", this);
    confirmSuppr->setChecked(pref.getConfirmSuppr());

    QVBoxLayout *layoutDivers = new QVBoxLayout;
    layoutDivers->addWidget(confirmSuppr, Qt::AlignTop);

    groupDivers->setLayout(layoutDivers);


    QHBoxLayout *hbox = new QHBoxLayout;
    hbox->addWidget(groupAffichage);
    hbox->addWidget(groupTri);

    // Layout Principal
    QVBoxLayout *layoutPrincipal = new QVBoxLayout;
    layoutPrincipal->addLayout(hbox);
    layoutPrincipal->addWidget(groupDivers);

    ongletGeneral->setLayout(layoutPrincipal);
}

void FenPreferences::activerOrdre(bool coche)
{
    // Si le tri auto est activé
    if (coche)
        ordre->setEnabled(true);
    // Sinon on désactive l'ordre de tri
    else
        ordre->setEnabled(false);
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

void FenPreferences::creerOngletInterros()
{
    // --- Notation ---
    QGroupBox *groupNotation = new QGroupBox("Notation", this);

    noteSur = new QSpinBox(this);
    noteSur->setRange(NOTESURMIN, NOTESURMAX);
    noteSur->setValue(pref.getNoteSur());

    commentaire = new QCheckBox("Activer le commentaire", this);
    commentaire->setChecked(pref.getCommentaire());

    QFormLayout *layoutNotation = new QFormLayout;
    layoutNotation->addRow("Noté sur : ", noteSur);
    layoutNotation->addRow(commentaire);

    groupNotation->setLayout(layoutNotation);


    // --- Options ---
    QGroupBox *groupOptions = new QGroupBox("Options", this);

    nbMots = new QSpinBox(this);
    nbMots->setRange(1, tailleDico);
    nbMots->setValue(pref.getNbMots());

    langQuestions = new QComboBox(this);
    langQuestions->addItem("Français");
    langQuestions->addItem("Anglais");
    langQuestions->addItem("Les deux");
    if (pref.getLangQuestions() == EN)
        langQuestions->setCurrentIndex(1);
    else if (pref.getLangQuestions() == BOTH)
        langQuestions->setCurrentIndex(2);

    QFormLayout *layoutOptions = new QFormLayout;
    layoutOptions->addRow("Nombre de questions : ", nbMots);
    layoutOptions->addRow("Langue des questions : ", langQuestions);

    groupOptions->setLayout(layoutOptions);


    // --- Correction ---
    QGroupBox *groupCorrection = new QGroupBox("Correction", this);

    parenthesesExclues = new QCheckBox("Exclure le contenu des parenthèses situées après l'expression", this);
    parenthesesExclues->setChecked(pref.getParenthesesExclues());

	toExclus = new QCheckBox("Exclure les \"to \" (verbe)", this);
    toExclus->setChecked(pref.getToExclus());

    respecterCasseInterros = new QCheckBox("Respecter la casse", this);
    respecterCasseInterros->setChecked(pref.getResperterCasse());

    confirmSiPasFinie = new QCheckBox("Confirmer la correction si l'interro n'est pas finie", this);
    confirmSiPasFinie->setChecked(pref.getConfirmSiPasFinie());

    QFormLayout *layoutCorrection = new QFormLayout;
    layoutCorrection->addRow(parenthesesExclues);
    layoutCorrection->addRow(toExclus);
    layoutCorrection->addRow(respecterCasseInterros);
    layoutCorrection->addRow(confirmSiPasFinie);

    groupCorrection->setLayout(layoutCorrection);


    // Layout principal
    QVBoxLayout *layoutPrincipalInt = new QVBoxLayout;
    layoutPrincipalInt->addWidget(groupNotation);
    layoutPrincipalInt->addWidget(groupOptions);
    layoutPrincipalInt->addWidget(groupCorrection);

    ongletInterros->setLayout(layoutPrincipalInt);
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

void FenPreferences::creerConteneur()
{
    // Création du QTabWidget
    QTabWidget *onglets = new QTabWidget(this);

    // Ajout des onglets au QTabWidget
	onglets->addTab(ongletGeneral, "Général");
	onglets->addTab(ongletInterros, "Interrogations");

    // Les boutons
    QPushButton *ok = new QPushButton("&OK", this);
    connect(ok, SIGNAL(clicked()), this, SLOT(valider()));
    QPushButton *annuler = new QPushButton("&Annuler", this);
    connect(annuler, SIGNAL(clicked()), this, SLOT(accept()));
    QPushButton *defaut = new QPushButton(" Préférences par défaut ", this);
    connect(defaut, SIGNAL(clicked()), this, SLOT(mettreDefaut()));

    QHBoxLayout *layoutBoutons = new QHBoxLayout;
    layoutBoutons->addWidget(ok);
    layoutBoutons->addWidget(annuler);
    layoutBoutons->addWidget(defaut);
    layoutBoutons->setAlignment(Qt::AlignRight);


    // Création du layout proncipal
    layoutPrincipal = new QVBoxLayout;
    layoutPrincipal->addWidget(onglets);
	layoutPrincipal->addLayout(layoutBoutons);
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

void FenPreferences::valider()
{
    ok = true;

    pref.save
    (
        colonneGauche->currentIndex(),
        largeurColonne->value(),
        tailleTexte->currentIndex(),
        triAuto->isChecked(),
        ordre->currentIndex(),
        respecterCasse->isChecked(),
        confirmSuppr->isChecked(),
        nbMots->value(),
        noteSur->value(),
        commentaire->isChecked(),
        respecterCasseInterros->isChecked(),
        confirmSiPasFinie->isChecked(),
        parenthesesExclues->isChecked(),
		toExclus->isChecked(),
		langQuestions->currentIndex()
    );

    accept();
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

void FenPreferences::mettreDefaut()
{
    QString msg = "Etes-vous sûr de vouloir réinitialiser <i>toutes</i> vos préférences à leur valeur par défaut ?";
    int reponse = QMessageBox::question(this, "Confirmation", msg, QMessageBox::Yes | QMessageBox::No);
    if (reponse == QMessageBox::Yes)
    {
        ok = true;
        pref.assignerDefauts();
        pref.save(tailleDico);
        accept();
    }

}

/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

Preferences FenPreferences::getPref()
{
    return pref;
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

bool FenPreferences::isOk()
{
    return ok;
}
