/*****************************************************************************************
**
** vEnglish - French-English Dictionary and more !
** Contact : vin.fouache@free.fr
**
** Copyright (C) 2008  Vincent Fouache
**
** This file is part of vEnglish.
**
** vEnglish is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation, either version 3 of the License, or
** (at your option) any later version.

** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>
**
***************************************************************************************/


/* ********** ********** ********** ********** ********** ********* *\
*
*               FenPrincipale.cc
*               ----------------
*
*   Auteur : Vincent Fouache
*   Date de création : 23 Octobre 2008
*   Dernières modifications : 09/12/08
*
*   Description :
*   Implémentation de la classe FenPrincipale
*
\* ********** ********** ********** ********** ********** ********* */

#include "FenPrincipale.hh"


/*
=================================================================================================
    Constructeur et méthodes de créations des menus, du formulaire d'ajout et du tableau
=================================================================================================
*/

FenPrincipale::FenPrincipale()
{
    // Initialisation
    modif = false;

    if (!pref.lire())
        QMessageBox::critical(this, "Erreur !", "Le fichier de configuration vEnglish.ini n'a pas pu être lu.<br />Les valeurs pas défaut seront utilisées.");

	// Création de la zone centrale
    zoneCentrale = new QWidget;
    zoneCentrale->setObjectName("zoneCentrale");

    // Création des widgets, layouts, styles...
    layoutPrincipal = new QVBoxLayout;
    creerTableau();
    creerFormulaire();
    creerActions();
    creerMenus();

    // Quelques propriétés pour la fenêtre
    setWindowTitle("vEnglish");
    setWindowIcon(QIcon(":/images/tudor_rose.png"));
    zoneCentrale->setLayout(layoutPrincipal);
    setCentralWidget(zoneCentrale);
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

void FenPrincipale::creerTableau()
{
    tableau = new QTableWidget;

    tableau->setRowCount(0);
    tableau->setColumnCount(2);

    (tableau->verticalHeader())->hide(); // Pas de titre pour les lignes
    tableau->setSelectionMode(QAbstractItemView::SingleSelection); // Pas de sélection multiple
    tableau->setSelectionBehavior(QAbstractItemView::SelectRows); // Sélection par ligne
    tableau->setAlternatingRowColors(true);
//    tableau->setFrameShape(QFrame::NoFrame); // Pas de bordure

    remplirTableau();

    appliquerPreferencesAffichage();

    layoutPrincipal->addWidget(tableau);

    connect(tableau, SIGNAL(cellChanged(int, int)), this, SLOT(modifierTableau(int, int)));
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

void FenPrincipale::creerFormulaire()
{
    fr = new QLineEdit(this);
    en = new QLineEdit(this);

    QLabel *frImg = new QLabel(this);
    frImg->setPixmap(QPixmap(":/images/icones/fr.png"));
    QLabel *enImg = new QLabel(this);
    enImg->setPixmap(QPixmap(":/images/icones/en.png"));

    QPushButton *ajouter = new QPushButton("&Ajouter", this);
    connect(ajouter, SIGNAL(clicked()), this, SLOT(ajouterLigne()));

    QGridLayout *layoutFormulaire = new QGridLayout;
    layoutFormulaire->addWidget(frImg, 0, 0);
    layoutFormulaire->addWidget(fr, 0, 1);
    layoutFormulaire->addWidget(enImg, 1, 0);
    layoutFormulaire->addWidget(en, 1, 1);
    layoutFormulaire->addWidget(ajouter, 2, 1, Qt::AlignRight);

    layoutPrincipal->addLayout(layoutFormulaire);
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

void FenPrincipale::creerActions()
{
    // Fichier
    actionRecharger = new QAction("&Recharger", this);
    actionRecharger->setShortcut(QKeySequence(QKeySequence::Refresh));
    actionRecharger->setIcon(QIcon(":/images/icones/reload.png"));
    connect(actionRecharger, SIGNAL(triggered()), this, SLOT(recharger()));

    actionSave = new QAction("&Enregistrer", this);
    actionSave->setShortcut(QKeySequence(QKeySequence::Save));
    actionSave->setIcon(QIcon(":/images/icones/filesave.png"));
    connect(actionSave, SIGNAL(triggered()), this, SLOT(save()));

    actionQuitter = new QAction("&Quitter", this);
    actionQuitter->setShortcut(QKeySequence("Ctrl+Q"));
    actionQuitter->setIcon(QIcon(":/images/icones/exit.png"));
    connect(actionQuitter, SIGNAL(triggered()), this, SLOT(quitter()));


    // Edition
    actionTrierAZ = new QAction("&Tri croissant", this);
    actionTrierAZ->setShortcut(QKeySequence("Ctrl+T"));
    actionTrierAZ->setIcon(QIcon(":/images/icones/sort_incr.png"));
    connect(actionTrierAZ, SIGNAL(triggered()), this, SLOT(trierAZ()));

    actionTrierZA = new QAction("&Tri décroissant", this);
    actionTrierZA->setShortcut(QKeySequence("Ctrl+SHIFT+T"));
    actionTrierZA->setIcon(QIcon(":/images/icones/sort_decr.png"));
    connect(actionTrierZA, SIGNAL(triggered()), this, SLOT(trierZA()));

    actionRechercher = new QAction("&Rechercher", this);
    actionRechercher->setShortcut(QKeySequence(QKeySequence::Find));
    actionRechercher->setIcon(QIcon(":/images/icones/search.png"));
    connect(actionRechercher, SIGNAL(triggered()), this, SLOT(rechercher()));

    actionSupprimer = new QAction("&Supprimer une ligne", this);
    actionSupprimer->setShortcut(QKeySequence("Delete"));
    actionSupprimer->setIcon(QIcon(":/images/icones/delete.png"));
    connect(actionSupprimer, SIGNAL(triggered()), this, SLOT(supprimerLigne()));

    actionPreferences = new QAction("&Préférences", this);
    actionPreferences->setIcon(QIcon(":/images/icones/configure.png"));
    connect(actionPreferences, SIGNAL(triggered()), this, SLOT(preferences()));


    // Interros
    actionGenInt = new QAction("&Générer une interro", this);
    actionGenInt->setShortcut(QKeySequence("Ctrl+G"));
    actionGenInt->setIcon(QIcon(":/images/icones/gear.png"));
    connect(actionGenInt, SIGNAL(triggered()), this, SLOT(genererInterro()));

    actionGenIntImp = new QAction("Génerer une interro &imprimable", this);
    actionGenIntImp->setShortcut(QKeySequence(QKeySequence::Print));
    actionGenIntImp->setIcon(QIcon(":/images/icones/print.png"));
    actionGenIntImp->setEnabled(false);


    // ?
    actionAide = new QAction("&Aide", this);
    actionAide->setShortcut(QKeySequence::HelpContents);
    actionAide->setIcon(QIcon(":/images/icones/help.png"));
    actionAide->setEnabled(false);

    actionAbout = new QAction("A &propos de vEnglish", this);
    actionAbout->setIcon(QIcon(":/images/icones/info.png"));
    connect(actionAbout, SIGNAL(triggered()), this, SLOT(about()));
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

void FenPrincipale::creerMenus()
{
    // Fichier
    QMenu *menuFichier = menuBar()->addMenu("&Fichier");
    menuFichier->addAction(actionRecharger);
    menuFichier->addAction(actionSave);
    menuFichier->addSeparator();
    menuFichier->addAction(actionQuitter);

    // Edition
    QMenu *menuEdition = menuBar()->addMenu("&Edition");
    menuEdition->addAction(actionTrierAZ);
    menuEdition->addAction(actionTrierZA);
    menuEdition->addAction(actionRechercher);
    menuEdition->addSeparator();
    menuEdition->addAction(actionSupprimer);
    menuEdition->addSeparator();
    menuEdition->addAction(actionPreferences);

    // Interros
    QMenu *menuInterro = menuBar()->addMenu("&Interrogations");
    menuInterro->addAction(actionGenIntImp);
    menuInterro->addAction(actionGenInt);

    // Aide
    QMenu *menuAide = menuBar()->addMenu("&?");
    menuAide->addAction(actionAide);
    menuAide->addSeparator();
    menuAide->addAction(actionAbout);
}






/*
=================================================================================================
    Méthodes gérant le fonctionnement basique du logiciel
=================================================================================================
*/

void FenPrincipale::remplirTableau()
{
    int erreur = dicos.fileToDicos(); // Initialisation des dicos
    if (erreur == 1)
    {
        QMessageBox::critical(this, "Erreur !", "Le dictionnaire n'a pas pu être lu.");
        this->close();
        return;
    }
    else if (erreur == 2)
        QMessageBox::warning(this, "Attention !", "Le dictionnaire contient un ou plusieurs terme(s) sans traduction.");

    actualiserTableau();
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

void FenPrincipale::actualiserTableau()
{
    // On créé autant de lignes qu'il y a de termes traduits
    tableau->setRowCount(dicos.taille());

    // On parcourt les 2 lists entièrement
    for (int i = 0 ; i < dicos.taille() ; i++)
    {
        // On récupère l'expression française puis l'expression anglaise
        QString fr = dicos.getFr(i);
        QString en = dicos.getEn(i);

        // On insère les mots dans la ligne correspondante.
        if (pref.getColGauche() == FR)
        {
            tableau->setItem(i, 0, new QTableWidgetItem(fr));
            tableau->setItem(i, 1, new QTableWidgetItem(en));
        }
        else
        {
            tableau->setItem(i, 1, new QTableWidgetItem(fr));
            tableau->setItem(i, 0, new QTableWidgetItem(en));
        }
    }
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

void FenPrincipale::modifierTableau(int ligne, int colonne)
{
    bool ok;

    // Si on a modifié un terme français
    if (colonne == 0 && pref.getColGauche() == FR)
        ok = dicos.setTrad(ligne, (tableau->item(ligne, colonne))->text(), FR);
    else if (colonne == 1 && pref.getColGauche() == EN)
        ok = dicos.setTrad(ligne, (tableau->item(ligne, colonne))->text(), FR);
    // Sinon on a modifié un terme anglais
    else if (colonne == 1 && pref.getColGauche() == FR)
        ok = dicos.setTrad(ligne, (tableau->item(ligne, colonne))->text(), EN);
    else if (colonne == 0 && pref.getColGauche() == EN)
        ok = dicos.setTrad(ligne, (tableau->item(ligne, colonne))->text(), EN);
    else
        ok = false;

    // Si la modification a échouée
    if (!ok)
    {
        QString msg = "Vous ne pouvez pas utiliser le caractère";
        msg += " <b>'" + QString(dicos.getSeparation()) + "'</b> ";
        msg += "car il est utilisé dans le fichier qui contient le dictionnaire ";
        msg += "comme caractère de séparation.";
        QMessageBox::warning(this, "Erreur !", msg);

        // On efface la cellule contenant le caractère de séparation
        tableau->setItem(ligne, colonne, new QTableWidgetItem(""));
    }
    else
        modif = true;
}


/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

void FenPrincipale::appliquerPreferencesAffichage()
{
    // Largeur des colonnes
    tableau->setColumnWidth(0, pref.getLargeurColonnes());
    tableau->setColumnWidth(1, pref.getLargeurColonnes());
    resize(pref.getLargeurColonnes() * 2 + 20, 400);

    // Taille du texte
    if (pref.getTailleTexte() == 0)
        tableau->setStyleSheet("QTableWidget { font-size: 6pt; }");
    else if (pref.getTailleTexte() == 1)
        tableau->setStyleSheet("QTableWidget { font-size: 8pt; }");
    else if (pref.getTailleTexte() == 2)
        tableau->setStyleSheet("QTableWidget { font-size: 10pt; }");
    else
        tableau->setStyleSheet("QTableWidget { font-size: 12pt; }");

    // Titre des colonnes
    if (pref.getColGauche() == FR)
    {
        tableau->setHorizontalHeaderItem(0, new QTableWidgetItem("Français"));
        tableau->setHorizontalHeaderItem(1, new QTableWidgetItem("Anglais"));
    }
    else
    {
        tableau->setHorizontalHeaderItem(1, new QTableWidgetItem("Français"));
        tableau->setHorizontalHeaderItem(0, new QTableWidgetItem("Anglais"));
    }

    // Tri automatique
    if (pref.getTriAuto())
        if (pref.getTriOrdre() == CROISSANT)
            trierAZ();
        else
            trierZA();
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

void FenPrincipale::keyPressEvent(QKeyEvent *event)
{
    // Les 2 touches "Entrée"
    if (event->key() == Qt::Key_Enter || event->key() == Qt::Key_Return)
        ajouterLigne();
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

void FenPrincipale::ajouterLigne()
{
    // On vérifie que le formulaire a été bien rempli.
    if (!validerFormulaire())
        return;

    // A partir de ce moment, il y a eu modification du dico
    // On demandera une confirmation avant de quitter
    modif = true;

    /* **********************************************************************
        Ajout des termes dans le tableau
    ********************************************************************** */

    // On ne modifie pas les lists.
    // C'est modifierTableau() qui s'en charge

    // Ici, je ne me sers pas de actualiserTableau(), J'ajoute simplement une ligne au tableau
    // ça évite de passer par une boucle...

    // On ajoute une ligne à la fin du tableau.
    tableau->setRowCount(tableau->rowCount() + 1);

    // On insère les termes dans la ligne fraichement créée.
    if (pref.getColGauche() == FR)
    {
        tableau->setItem(tableau->rowCount() - 1, 0, new QTableWidgetItem(fr->text()));
        tableau->setItem(tableau->rowCount() - 1, 1, new QTableWidgetItem(en->text()));
    }
    else
    {
        tableau->setItem(tableau->rowCount() - 1, 1, new QTableWidgetItem(fr->text()));
        tableau->setItem(tableau->rowCount() - 1, 0, new QTableWidgetItem(en->text()));
    }


    // On réinitialise les champs.
    fr->setText("");
    en->setText("");
    fr->setFocus();
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

bool FenPrincipale::validerFormulaire()
{
	bool pasErreur = false;

    // Si le français n'a pas été indiqué
    if (fr->text().isEmpty())
        QMessageBox::warning(this, "Formulaire incomplet", "Vous n'avez pas indiqué le terme français.");

    // Si l'anglais n'a pas été indiqué
    else if (en->text().isEmpty())
        QMessageBox::warning(this, "Formulaire incomplet", "Vous n'avez pas indiqué le terme anglais.");

    else
    	pasErreur = true;

    return pasErreur;
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

// Quand on clique sur la croix rouge
void FenPrincipale::closeEvent(QCloseEvent* event)
{
    quitter();
}






/*
=================================================================================================
    Méthodes du menu Fichier
=================================================================================================
*/

void FenPrincipale::recharger()
{
    if (modif)
    {
        int reponse = QMessageBox::question(this, "Confirmation", "Toutes vos modifications seront perdues.<br />Voulez-vous vraiment recharger le dictionnaire ?", QMessageBox::Yes | QMessageBox::No);

        if (reponse == QMessageBox::Yes)
            remplirTableau();
    }
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

void FenPrincipale::save()
{
    dicos.toFile();
    modif = false;
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

void FenPrincipale::quitter()
{
    // Si il y a eu des modifs, on propose d'enregistrer
    if (modif)
    {
        int reponse = QMessageBox::question(this, "Confirmation", "Voulez-vous enregistrer votre dictionnaire avant de quitter ?", QMessageBox::Yes | QMessageBox::No);

        if (reponse == QMessageBox::Yes)
            save();
    }

    qApp->quit();
}





/*
=================================================================================================
    Méthodes du menu Edition
=================================================================================================
*/

void FenPrincipale::trierAZ()
{
    dicos.trier(CROISSANT, pref.getTriCasse());
    actualiserTableau();
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

void FenPrincipale::trierZA()
{
    dicos.trier(DECROISSANT, pref.getTriCasse());
    actualiserTableau();
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

void FenPrincipale::supprimerLigne()
{
    int reponse;

    // Si la confirmation de suppression est activée
    if (pref.getConfirmSuppr())
    {
        QString msg = "Supprimer la ligne : <b>";
        if (pref.getColGauche() == FR)
            msg += dicos.getFr(tableau->currentRow()) + " : " + dicos.getEn(tableau->currentRow());
        else
            msg += dicos.getEn(tableau->currentRow()) + " : " + dicos.getFr(tableau->currentRow());
        msg += "</b> ?";

        reponse = QMessageBox::question(this, "Confirmation", msg, QMessageBox::Yes | QMessageBox::No);
    }
    // Sinon on répond Oui automatiquement.
    else
        reponse = QMessageBox::Yes;

    // Si l'utilisateur a cliqué sur Oui ou si la confirmation est désactivée
    if (reponse == QMessageBox::Yes)
    {
        // On supprime les entrées des dicos.
        dicos.delTrad(tableau->currentRow(), FR);
        dicos.delTrad(tableau->currentRow(), EN);

        // On affiche le tableau avec la ligne en moins.
        actualiserTableau();
    }
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

void FenPrincipale::rechercher()
{
    FenRecherche *dialogRecherche = new FenRecherche(dicos, this);
    dialogRecherche->exec();
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

void FenPrincipale::preferences()
{
    FenPreferences *prefer = new FenPreferences(pref, dicos.taille(), this);
    prefer->exec();

    // Si l'utilisateur a cliqué sur OK ou sur par défaut
    if (prefer->isOk())
    {
        // On récupère les nouvelles préférences.
        pref = prefer->getPref();

        // On applique le nouvel affichage.
        appliquerPreferencesAffichage();

        // On récupère la valeur de modif
        int modifTmp = modif;
        // On actualise l'affichage
        actualiserTableau(); // Maintenant modif == true, car on est passé par modifierTableau
        // Si la valeur de modif avant l'actualisation était false
        // alors on remet sa valeur à false.
        // Ca évite d'avoir un avertissement "Voulez-vs sauver av de quitter?" alors que l'on a rien modifié
        if (modifTmp == false)
            modif = false;
    }
}






/*
=================================================================================================
    Méthodes du menu Interros
=================================================================================================
*/

void FenPrincipale::genererInterro()
{
    if (pref.getNbMots() > dicos.taille())
        QMessageBox::critical(this, "Erreur !", "Le nombre de questions de l'interrogation est supérieur au nombre de termes dans le dictionnaire !");
    else
    {
        // On cache le dico :p
        tableau->setVisible(false);

        FenInterro *interro = new FenInterro(dicos, pref);
        interro->exec();

        // On réaffiche le dico
        tableau->setVisible(true);
    }
}






/*
=================================================================================================
    Méthodes du menu ?
=================================================================================================
*/

void FenPrincipale::about()
{
    FenAbout *aPropos = new FenAbout(this);
    aPropos->exec();
}
