/*****************************************************************************************
**
** vEnglish - French-English Dictionary and more !
** Contact : vin.fouache@free.fr
**
** Copyright (C) 2008  Vincent Fouache
**
** This file is part of vEnglish.
**
** vEnglish is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation, either version 3 of the License, or
** (at your option) any later version.

** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>
**
***************************************************************************************/


/* ********** ********** ********** ********** ********** ********* *\
*
*               FenRecherche.cc
*               ---------------
*
*   Auteur : Vincent Fouache
*   Date de création : 27 Octobre 2008
*   Dernières modifications : 31/10/08
*
*   Description : Implémentation de la classe FenRecherche
*
\* ********** ********** ********** ********** ********** ********* */

#include "FenRecherche.hh"


FenRecherche::FenRecherche(const Dicos &dicosP, QWidget *parent = 0) : QDialog(parent)
{
    // Définition de l'attribut dicos
    dicos = dicosP;

    // --- Texte à rechercher ---
    recherche = new QLineEdit(this);
    QFormLayout *layoutRecherche = new QFormLayout;
    layoutRecherche->addRow("Texte à rechercher : ", recherche);



    // --- Options ---
    QGroupBox *groupOptions = new QGroupBox("Options", this);
    // Où la recherche sera effectuée
    lieu = new QComboBox(this);
    lieu->addItem("le français");
    lieu->addItem("l'anglais");
    lieu->addItem("l'anglais et le français");
    lieu->setCurrentIndex(2);

    // Activer la casse
    casse = new QCheckBox("Respecter la casse", this);

    QFormLayout *layoutOptions = new QFormLayout;
    layoutOptions->addRow("Rechercher dans : ", lieu);
    layoutOptions->addRow(casse);
    groupOptions->setLayout(layoutOptions);



    // --- La recherche correspond à ---
    QGroupBox *groupRadio = new QGroupBox("La recherche correspond à", this);
    partieMot = new QRadioButton("une partie d'un mot");
    entierMot = new QRadioButton("un mot entier", this);
    debutMot = new QRadioButton("un début de mot", this);
    finMot = new QRadioButton("une fin de mot", this);
    partieMot->setChecked(true);

    QVBoxLayout *layoutRadio = new QVBoxLayout;
    layoutRadio->addWidget(partieMot);
    layoutRadio->addWidget(entierMot);
    layoutRadio->addWidget(debutMot);
    layoutRadio->addWidget(finMot);
    groupRadio->setLayout(layoutRadio);


    // Boutons OK et Annuler
    QPushButton *ok = new QPushButton("&OK", this);
    connect(ok, SIGNAL(clicked()), this, SLOT(rechercher()));
    QPushButton *annuler = new QPushButton("&Annuler", this);
    connect(annuler, SIGNAL(clicked()), this, SLOT(accept()));

    QHBoxLayout *layoutBouton = new QHBoxLayout;
    layoutBouton->addWidget(ok);
    layoutBouton->addWidget(annuler);
    layoutBouton->setAlignment(Qt::AlignRight);


    // Layout principal
    QVBoxLayout *layoutPrincipal = new QVBoxLayout;
    layoutPrincipal->addLayout(layoutRecherche);
    layoutPrincipal->addWidget(groupOptions);
    layoutPrincipal->addSpacing(10);
    layoutPrincipal->addWidget(groupRadio);
    layoutPrincipal->addLayout(layoutBouton);


    // Quelques propriétés pour la fenêtre
    setLayout(layoutPrincipal);
    setWindowTitle("Rechercher");
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

void FenRecherche::rechercher()
{
    if ((recherche->text()).isEmpty())
        QMessageBox::warning(this, "Erreur !", "Vous n'avez rien indiqué dans le champ de recherche.");
    else
    {
        // Initialisation du résultat
        resultat = "";
        int nbResultats;


        // Préparation de la regex
        regex.setPatternSyntax(QRegExp::Wildcard);

        if (casse->isChecked())
            regex.setCaseSensitivity(Qt::CaseSensitive);
        else
            regex.setCaseSensitivity(Qt::CaseInsensitive);

        if (partieMot->isChecked())
            regex.setPattern("*" + recherche->text() + "*");
        else if (entierMot->isChecked())
            regex.setPattern(recherche->text());
        else if (debutMot->isChecked())
            regex.setPattern(recherche->text() + "*");
        else if (finMot->isChecked())
            regex.setPattern("*" + recherche->text());


        // Si on doit effectuer la recherche ds le dico fr
        if (lieu->currentIndex() == 0)
            nbResultats = index(FR);
        // Si on doit effectuer la recherche ds le dico en
        else if (lieu->currentIndex() == 1)
            nbResultats = index(EN);
        // Si on doit effectuer la recherche ds les 2 dicos
        else
        {
            nbResultats = index(FR);
            nbResultats += index(EN);
        }

        // Affichage du résultat
        if (nbResultats == 1)
            QMessageBox::information(this, "1 résultat pour " + recherche->text(), resultat);
        else if (nbResultats > 1)
            QMessageBox::information(this, QString::number(nbResultats) + " résultats pour " + recherche->text(), resultat);
        else
            QMessageBox::information(this, "Résultat de la recherche", "Pas de résultat pour : <i>" + recherche->text() + "</i>");

        accept();
    }
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

int FenRecherche::index(Langue lang)
{
    int nbResultats = 0, i = 0;

    // Si la recherche n'aboutit pas alors indexOf renvoie -1
    while (i != -1)
    {
        i = dicos.indexOf(regex, i, lang);

        // Si on a trouvé quelque chose
        if (i != -1)
        {
            nbResultats++;
            resultat += dicos.getFr(i) + " : " + dicos.getEn(i) + "<br />";

            // Pour ne pas dépasser la liste...
            if (i < dicos.taille())
                i++;
            else
                i = -1;
        }
    }

    return nbResultats;
}
