/*****************************************************************************************
**
** vEnglish - French-English Dictionary and more !
** Contact : vin.fouache@free.fr
**
** Copyright (C) 2008  Vincent Fouache
**
** This file is part of vEnglish.
**
** vEnglish is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation, either version 3 of the License, or
** (at your option) any later version.

** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>
**
***************************************************************************************/


/* ********** ********** ********** ********** ********** ********* *\
*
*               Preferences.cc
*               --------------
*
*   Auteur : Vincent Fouache
*   Date de création : 31 Octobre 2008
*   Dernières modifications : 05/11/08
*
*   Description : Implémentation de la classe Preferences
*
\* ********** ********** ********** ********** ********** ********* */

#include "Preferences.hh"


Preferences::Preferences()
{
    assignerDefauts();
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

void Preferences::assignerDefauts()
{
    // Général
    p_colGauche = FR;
    p_largeurColonnes = 150;
    p_tailleTexte = 1;
    p_triAuto = false;
    p_triOrdre = CROISSANT;
    p_triCasse = false;
    p_confirmSuppr = true;

    // Interros
    p_nbMots = 1;
    p_noteSur = 20;
    p_commentaire = true;
    p_respecterCasse = false;
    p_confirmSiPasFinie = true;
    p_parenthesesExclues = true;
    p_toExclus = true;
    p_langQuestions = FR;
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

Langue Preferences::getColGauche() const
{
    return p_colGauche;
}
int Preferences::getLargeurColonnes() const
{
    return p_largeurColonnes;
}
int Preferences::getTailleTexte() const
{
    return p_tailleTexte;
}
bool Preferences::getTriAuto() const
{
    return p_triAuto;
}
Tri Preferences::getTriOrdre() const
{
    return p_triOrdre;
}
bool Preferences::getTriCasse() const
{
    return p_triCasse;
}
bool Preferences::getConfirmSuppr() const
{
    return p_confirmSuppr;
}
int Preferences::getNbMots() const
{
    return p_nbMots;
}
int Preferences::getNoteSur() const
{
    return p_noteSur;
}
bool Preferences::getCommentaire() const
{
    return p_commentaire;
}
bool Preferences::getResperterCasse() const
{
    return p_respecterCasse;
}
bool Preferences::getConfirmSiPasFinie() const
{
    return p_confirmSiPasFinie;
}
bool Preferences::getParenthesesExclues() const
{
    return p_parenthesesExclues;
}
bool Preferences::getToExclus() const
{
    return p_toExclus;
}
Langue Preferences::getLangQuestions() const
{
    return p_langQuestions;
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

bool Preferences::lire()
{
    bool pasErreur = true;

    /* **********************************************************************
        Ouverture du fichier
    ********************************************************************** */

    QFile file("vEnglish.ini");
    // Si le fichier n'a pas pu être ouvert
    // Les valeurs restent donc par défaut.
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
        pasErreur = false;
    else
    {
        /* **********************************************************************
            Lecture du fichier, récupération des valeurs
        ********************************************************************** */

        QTextStream in(&file);

        p_colGauche = toLangue(in.readLine().toInt());
        p_largeurColonnes = comprisEntre(LGCOLMIN, LGCOLMAX, in.readLine().toInt());
        p_tailleTexte = comprisEntre(SIZETEXTMIN, SIZETEXTMAX, in.readLine().toInt());
        p_triAuto = toBool(in.readLine().toInt());
        p_triOrdre = toTri(in.readLine().toInt());
        p_triCasse = toBool(in.readLine().toInt());
        p_confirmSuppr = toBool(in.readLine().toInt());
        p_nbMots = in.readLine().toInt();
        p_noteSur = comprisEntre(NOTESURMIN, NOTESURMAX, in.readLine().toInt());
        p_commentaire = toBool(in.readLine().toInt());
        p_respecterCasse = toBool(in.readLine().toInt());
        p_confirmSiPasFinie = toBool(in.readLine().toInt());
        p_parenthesesExclues = toBool(in.readLine().toInt());
        p_toExclus = toBool(in.readLine().toInt());
        p_langQuestions = toLangue(in.readLine().toInt());


        // On ferme le fichier.
        file.close();
    }

    return pasErreur;
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

// Enregistrer dans le fichier toutes les valeurs actuelles des attributs
// Sauf nbMots dont la valeur est récupérée en paramètre
void Preferences::save(int nbMots)
{
    p_nbMots = nbMots;

    save
    (
        p_colGauche,
        p_largeurColonnes,
        p_tailleTexte,
        p_triAuto,
        p_triOrdre,
        p_triCasse,
        p_confirmSuppr,
        p_nbMots,
        p_noteSur,
        p_commentaire,
        p_respecterCasse,
        p_confirmSiPasFinie,
        p_parenthesesExclues,
		p_toExclus,
		p_langQuestions
    );
}

void Preferences::save
(
    int colGauche, int largeurColonnes, int tailleTexte,
    bool triAuto, int triOrdre, bool triCasse,
    bool confirmSuppr,
    int nbMots, int noteSur, bool commentaire,
    bool respecterCasse, bool confirmSiPasFinie,
    bool parenthesesExclues, bool toExclus, int langQuestions
)
{
    // Ouverture du fichier
    QFile file("vEnglish.ini");
    file.open(QIODevice::WriteOnly | QIODevice::Text);


    /* **********************************************************************
        Ecriture du fichier
    ********************************************************************** */

     QTextStream out(&file);
     out << colGauche << endl << largeurColonnes << endl << tailleTexte << endl;
     out << triAuto  << endl << triOrdre << endl << triCasse << endl;
     out << confirmSuppr << endl;
     out << nbMots << endl << noteSur << endl << commentaire << endl;
     out << respecterCasse << endl << confirmSiPasFinie << endl;
     out << parenthesesExclues << endl << toExclus << endl << langQuestions;

    // On ferme le fichier.
    file.close();

    /* **********************************************************************
        Enregistrement des attributs
    ********************************************************************** */

    p_colGauche = toLangue(colGauche);
    p_largeurColonnes = largeurColonnes;
    p_tailleTexte = tailleTexte;
    p_triAuto = toBool(triAuto);
    p_triOrdre = toTri(triOrdre);
    p_triCasse = toBool(triCasse);
    p_confirmSuppr = toBool(confirmSuppr);
    p_nbMots = nbMots;
    p_noteSur = noteSur;
    p_commentaire = toBool(commentaire);
    p_respecterCasse = toBool(respecterCasse);
    p_confirmSiPasFinie = toBool(confirmSiPasFinie);
    p_parenthesesExclues = toBool(parenthesesExclues);
    p_toExclus = toBool(toExclus);
    p_langQuestions = toLangue(langQuestions);
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

Langue Preferences::toLangue(const int x)
{
    Langue lang;
    if (x == 1)
        lang = EN;
    else if (x == 2)
        lang = BOTH;
    else
        lang = FR; // défaut
    return lang;
}

Tri Preferences::toTri(const int x)
{
    Tri ordre;
    if (x == 1)
        ordre = DECROISSANT;
    else
        ordre = CROISSANT; // défaut
    return ordre;
}

bool Preferences::toBool(const int x)
{
    bool res = true; // defaut
    if (x == 0)
        res = false;
    return res;
}

int Preferences::comprisEntre(const int a, const int b, const int nb)
{
    int res = nb;
    if (nb < a)
        res = a;
    else if (nb > b)
        res = b;
    return res;
}
